# Soya 3D tutorial
# Copyright (C) 2001-2002 Bertrand 'blam!' LAMY
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


# ----------------------------------------
# Lesson 117: Advanced Landscape functions
# ----------------------------------------

# This lesson shows some advanced landscape feature:
#   - landscape static lighting
#   - landscape visibility to perform a 'fog of war' effect
#   - getting the height of the land instead of using raypicking (faster but less powerfull)

import os, os.path, sys, time

import soya.model
import soya.soya3d
import soya.land


soya.init()

soya.model.Image.PATH = os.path.join(os.path.dirname(os.path.abspath(sys.argv[0])), "data")

# Create a world
world = soya.soya3d.World()

# Add a light
light = soya.soya3d.Light(world)
light.ambient = (0.2, 0.2, 0.2, 1.0)
light.set_xyz(0.0, 15.0, 0.0)

# Create a land
land = soya.land.Land(0)
land.from_image(soya.model.Image("map1.tga"))
land.multiply_height(8.0)
# Texture the land
material1 = soya.model.Material()
material1.tex_filename  = "block2.tga"
material2 = soya.model.Material()
material2.tex_filename  = "metal1.tga"
land.set_texture_layer(material1, 0.0, 6.0)
land.set_texture_layer(material2, 6.0, 8.0)

world.set_shape(land)


# Land static lighting
# --------------------
# compute_static_light(LIGHTS, SHADOW, SMOOTH)
#
# LIGHTS: a list of lights. All lights must be defined (ie added) into the
#   land coordsys (ie the volume or world that have the land as shape).
# SHADOW: negative value if you want no shadow else this is the number of
#   time the shadows are smoothed. This argument is optional (default -1).
# SMOOTH: this is a float factor that represent the conservation of the
#   shadow value when smoothing (1.0 is no smooth). This argument must only
#   given if SHADOW is given and is optional (default value 0.5)
#
# To remove static lights from a land use:
#   land.compute_static_light()

land.compute_static_light([light], -1)

world.remove(light)


# Setting land visibility
# -----------------------
#   Each point of the land can be visible or invisible. By default all the
#   points are visible.
#   You can use the following functions (0 is invisible and 1 is visible):

x = 0
z = 2
land.set_point_visibility(x, z, 0)
visibility = land.get_point_visibility(x, z)
# (x and z must be Integer)

# Set all the points of the land invisible
land.set_all_points_visibility(0)

# And now the most interesting functions:
#
# land.set_point_visibility_in_sphere(VISIBILITY, CENTER_X, CENTER_Y, CENTER_Z, RADIUS)
#   (CENTER_X, CENTER_Y, CENTER_Z, RADIUS must be given in the coordsys of the land)
#
# land.set_point_visibility_in_cylinder(VISIBILITY, CENTER_X, CENTER_Z, RADIUS)
#
# As you expect it, these 2 function set the visibility of all the points that are inside
# the given volume (sphere or cylinder). The cylinder is always oriented along the Y axis.



# Add a camera and a loop to render
camera = soya.soya3d.Camera(world)
camera.set_xyz (0.0, 16.0, 0.0)
camera.turn_vertical(-90.0)
soya.set_root_widget(camera)

while(1):

# Reveal the part of the land that is around the camera
  land.set_point_visibility_in_cylinder(1, camera.x, camera.z, 5.0)
  
  soya.render()

  time.sleep(0.1)
  
# Use Up, Down, Left, Right arrows to move the camera

  for event in soya.process_event():
    if event[0] == soya.KEYDOWN:
      if event[1] == soya.K_UP:
        camera.translate(0.0, 0.0, -0.5)
      elif event[1] == soya.K_DOWN:
        camera.translate(0.0, 0.0, 0.5)
      elif event[1] == soya.K_LEFT:
        camera.translate(-0.5, 0.0, 0.0)
      elif event[1] == soya.K_RIGHT:
        camera.translate(0.5, 0.0, 0.0)
      elif event[1] == soya.K_q:
        sys.exit()

# Landscape get_true_height
# -------------------------
#
# height = get_true_height(x, z)
#   This function will return the height associated with the position x and z.
#   x and z must be Float and given in the land coordsys. The height is interpolated
#   (this is different from the get_height function).
#
# (height, normal) = get_true_height_normal(x, z)
#   Same as above but return a tuple with the height and the point normal associated
#   with the position (x, z). Notice that normal is a 3 floats tuple, not a Vector
#   cause the land is a shape that doesn't have a parent and so doesn't know its coordsys.

      camera.y = land.get_true_height(camera.x, camera.z) + 16.0


# Conclusion
# ----------
#  Now that you know all of this you can use this 'fog of war' effect to create a wargame
#  or real time strategy game (like Warcraft III... ;). Anybody interested?
    


